<?php

namespace SGalinski\SgNews\Controller;

/***************************************************************
 *  Copyright notice
 *
 *  (c) sgalinski Internet Services (https://www.sgalinski.de)
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use SGalinski\SgNews\Domain\Model\News;
use SGalinski\SgNews\Domain\Repository\CategoryRepository;
use SGalinski\SgNews\Domain\Repository\NewsRepository;
use SGalinski\SgNews\Domain\Repository\TagRepository;
use SGalinski\SgNews\Domain\Service\NewsService;
use SGalinski\SgNews\Service\ConfigurationService;
use SGalinski\SgNews\Service\HeaderMetaDataService;
use TYPO3\CMS\Core\Http\ImmediateResponseException;
use TYPO3\CMS\Core\Utility\ExtensionManagementUtility;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Frontend\Controller\ErrorController;
use TYPO3\CMS\Frontend\Page\PageAccessFailureReasons;

/**
 * Controller that handles the list rendering of a single category
 */
class ListByCategoryController extends AbstractController {
	/**
	 * @var CategoryRepository
	 */
	protected $categoryRepository;

	/**
	 * @var TagRepository
	 */
	protected $tagRepository;

	/**
	 * @var NewsRepository
	 */
	protected $newsRepository;

	/**
	 * @var NewsService
	 */
	protected $newsService;

	/**
	 * @param CategoryRepository $categoryRepository
	 */
	public function injectCategoryRepository(
		CategoryRepository $categoryRepository
	) {
		$this->categoryRepository = $categoryRepository;
	}

	/**
	 * @param NewsRepository $newsRepository
	 */
	public function injectNewsRepository(NewsRepository $newsRepository) {
		$this->newsRepository = $newsRepository;
	}

	/**
	 * @param TagRepository $tagRepository
	 */
	public function injectTagRepository(TagRepository $tagRepository) {
		$this->tagRepository = $tagRepository;
	}

	/**
	 * @param NewsService $newsService
	 */
	public function injectNewsService(NewsService $newsService) {
		$this->newsService = $newsService;
	}

	/**
	 * Initialize the indexAction to set the currentPageBrowserPage parameter
	 *
	 * @throws \TYPO3\CMS\Extbase\Mvc\Exception\InvalidArgumentNameException
	 */
	public function initializeIndexAction() {
		$currentPageBrowserPage = GeneralUtility::_GP('tx_sgnews_pagebrowser')
			? (int) GeneralUtility::_GP('tx_sgnews_pagebrowser')['currentPage']
			: 0;
		if ($currentPageBrowserPage > 0) {
			$this->request->setArgument('currentPageBrowserPage', $currentPageBrowserPage);
		}
	}

	/**
	 * Renders the news list of a category
	 *
	 * @param array $newsMetaData
	 * @param int $currentPageBrowserPage
	 * @return null|\Psr\Http\Message\ResponseInterface
	 * @throws \InvalidArgumentException
	 * @throws \TYPO3\CMS\Extbase\Persistence\Exception\InvalidQueryException
	 * @throws \TYPO3\CMS\Extbase\Configuration\Exception\InvalidConfigurationTypeException
	 * @throws ImmediateResponseException
	 * @throws \TYPO3\CMS\Core\Error\Http\PageNotFoundException
	 */
	public function indexAction(array $newsMetaData = [], int $currentPageBrowserPage = 0): ?\Psr\Http\Message\ResponseInterface {
		$filterByCategories = FALSE;
		$categoryUids = [];
		$tagUids = [];
		if (isset($this->settings['categories'])) {
			$categoryUids = GeneralUtility::intExplode(',', $this->settings['categories'], TRUE);
		}
		if (isset($this->settings['tags'])) {
			$tagUids = GeneralUtility::intExplode(',', $this->settings['tags'], TRUE);
		}

		HeaderMetaDataService::addPageNumberToCanonical($currentPageBrowserPage);

		foreach ($categoryUids as $categoryUid) {
			if ($categoryUid > 0) {
				$filterByCategories = TRUE;
				break;
			}
		}

		if (!$filterByCategories) {
			$categoryUids = [(int) $GLOBALS['TSFE']->id];
		}

		$categories = [];
		foreach ($categoryUids as $categoryUid) {
			$category = $this->categoryRepository->findByUid($categoryUid);
			if (!$category) {
				continue;
			}

			$categories[$categoryUid] = $category;
			if ($category->_getProperty('_languageUid') > 0) {
				$originalLangCategory = $this->categoryRepository->findOriginalLanguageById($category->getUid());
				if ($originalLangCategory) {
					$categoriesById[$originalLangCategory->getUid()] = $originalLangCategory;
				}
			}
		}

		$startTime = 0;
		$endTime = 0;
		if (isset($this->settings['starttime'])) {
			$startTime = (int) $this->settings['starttime'];
		}
		if (isset($this->settings['endtime'])) {
			$endTime = (int) $this->settings['endtime'];
		}
		$newsPerPage = (int) $this->settings['newsLimitPerPage'];

		$newsCount = $this->newsRepository->newsCountByCategories($categoryUids, $tagUids, $startTime, $endTime);
		$numberOfPages = (int) ($newsPerPage <= 0 ? 0 : ceil($newsCount / $newsPerPage));
		if ($numberOfPages !== 0 && $currentPageBrowserPage >= $numberOfPages) {
			/** @var ErrorController $errorController */
			$errorController = GeneralUtility::makeInstance(ErrorController::class);
			$response = $errorController->pageNotFoundAction(
				$GLOBALS['TYPO3_REQUEST'],
				'The requested page does not exist',
				['code' => PageAccessFailureReasons::PAGE_NOT_FOUND]
			);
			throw new ImmediateResponseException($response);
		}

		$headerSet = FALSE;
		$offset = $this->calculatePaginationOffset($currentPageBrowserPage, $newsPerPage);

		$configurationService = GeneralUtility::makeInstance(ConfigurationService::class);
		$sortBy = $configurationService->getConfiguration('sortBy', $this->settings);
		$sortDirection = $configurationService->getConfiguration('sortDirection', $this->settings);

		$news = $this->newsRepository->findAllSortedNewsByCategories(
			$categoryUids,
			$newsPerPage,
			$offset,
			$sortBy,
			$tagUids,
			$startTime,
			$endTime,
			$sortDirection
		)->toArray();

		foreach ($news as $newsEntry) {
			/** @var News $newsEntry */
			$data = $this->newsService->getMetaDataForNews($newsEntry, $categories[$newsEntry->getPid()]);
			$newsMetaData[] = $data;
		}

		$this->view->assign('numberOfPages', $numberOfPages);
		$this->view->assign('newsMetaData', $newsMetaData);
		$this->view->assign('categories', $categories);

		if (version_compare(\TYPO3\CMS\Core\Utility\VersionNumberUtility::getCurrentTypo3Version(), '11.0.0', '<')) {
			return NULL;
		} else {
			return $this->htmlResponse();
		}
	}
}
